/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * This class extends DefaultHandler and is used to parse xml content.
 * It is used by SAX parser and is only used parse document collection xml.
 * </p>
 *
 * <p>
 * Note, SAX is used for loading xml file at the sake of speed and memory compared to DOM based parser.
 * </p>
 *
 * <p>
 * The following is a sample of document collection xml.
 * &lt;?xml version=&quot;1.0&quot;?&gt;
 * &lt;document-collection collection-id=&quot;37684&quot;&gt;
 *     &lt;documents&gt;
 *         &lt;document-id&gt;16572&lt;/document-id&gt;
 *         &lt;document-id&gt;46537&lt;/document-id&gt;
 *     &lt;/documents&gt;
 * &lt;/document-collection&gt;
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 *
 */
final class CollectionIndexHandler extends DefaultHandler {
    /**
     * <p>
     * Stores the value of collection id attribute in document collection xml file.
     * </p>
     */
    private String collectionId;

    /**
     * <p>
     * Sores the values of all document-id element in document collection xml file.
     * </p>
     */
    private List documentIdList;

    /**
     * <p>
     * Represents the current handled element name.
     * </p>
     */
    private String currentElementName;

    /**
     * <p>
     * Represents the value of currentElementName.
     * Note, an element value may be splitted into several character parts, so this field is declared as a member
     * to concatenate all the characters.
     * </p>
     */
    private StringBuffer content;

    /**
     * <p>
     * Creates a CollectionIndexHandler instance.
     * </p>
     *
     * <p>
     * Note, it's package private.
     * </p>
     */
    CollectionIndexHandler() {
        this.documentIdList = new ArrayList();
        content = new StringBuffer();
    }

    /**
     * <p>
     * Clears the content buffer in order to receive the current element text.
     * When the name of the current element is document-collection, the collection-id attribute is parsed.
     * </p>
     *
     * <p>
     * Note, when the SAX parser encounters a star-tag, this method is called.
     * </p>
     *
     * @param namespaceURI namespace uri
     * @param localName local name
     * @param qName qualified name
     * @param attr represents the attributes for the current element encountered
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String namespaceURI, String localName,
        String qName, Attributes attr) throws SAXException {
        currentElementName = qName.intern();

        content.delete(0, content.length());

        //when document-collection element is encountered, then save the collection-id attribute
        if (currentElementName == Constants.DOCUMENT_COLLECTION) {
            collectionId = attr.getValue(Constants.COLLECTION_ID);
        }
    }

    /**
     * <p>
     * If document-id element ends, then stores the element text.
     * This method will clear the currentElementName and clear the element content buffer.
     * </p>
     *
     * <p>
     * Note, when the SAX parser encounters a end-tag, this method is called.
     * </p>
     *
     * @param namespaceURI namespace uri
     * @param localName local name
     * @param qName qualified name
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void endElement(String namespaceURI, String localName, String qName)
        throws SAXException {

        //when document-id is encountered, then save the document-id element value
        if (currentElementName == Constants.DOCUMENT_ID) {
            documentIdList.add(content.toString());
        }

        //clear environment
        content.delete(0, content.length());
        currentElementName = null;
    }

    /**
     * <p>
     * Buffers all the characters for the current element in the xml file.
     * </p>
     *
     * <p>
     * Note, when the SAX parser reads #PDATA, this method is called.
     * </p>
     *
     * @param ch The characters.
     * @param start The start position in the character array.
     * @param length The number of characters to use from the character array.
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void characters(char[] ch, int start, int length)
        throws SAXException {
        //note, the text value for an element may be splitted to several characters
        //so, content buffer is used.
        content.append(new String(ch, start, length));
    }

    /**
     * <p>
     * Returns the collection id attribute in the current document collection xml file.
     * </p>
     *
     * @return the collection id attribute in the current document collection xml file.
     */
    String getCollectionId() {
        return collectionId;
    }

    /**
     * <p>
     * Returns all the document index id contained in the current document collection xml file.
     * </p>
     *
     * <p>
     * Note, the element in the returned list is type of String.
     * </p>
     *
     * @return all the document index id contained in the current document collection xml file.
     */
    List getDocumentIdList() {
        return documentIdList;
    }
}
