/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.DocumentIndexPersistenceUtil;
import com.topcoder.document.index.wordsource.WordSourceId;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.text.CollationKey;
import java.text.Collator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;


/**
 * <p>
 * This class extends DefaultHandler and is used to parse xml content.
 * It is used by SAX parser and is only used parse document index xml.
 * </p>
 *
 * <p>
 * Note, SAX is used for loading xml file at the sake of speed and memory compared to DOM based parser.
 * </p>
 *
 * <p>
 * The following is a sample of document index xml.
 * &lt;?xml version=&quot;1.0&quot;?&gt;
 * &lt;doc xml:lang=&quot;en-US&quot;&gt;
 *     &lt;document-index document-id=&quot;16527&quot;&gt;
 *         &lt;word-source-data&gt;
 *             &lt;source-identity&gt;rO0ABXQAE3Rlc3RfZmlsZXMvdGV4dC50eHQ=&lt;/source-identity&gt;
 *             &lt;class-name&gt;class name&lt;/class-name&gt;
 *             &lt;locale language=&quot;en&quot; country=&quot;US&quot; variant=&quot;POSIX&quot;/&gt;
 *             &lt;delimiters&gt;
 *                  &lt;entry delimiter=&quot;0x20&quot;/&gt;
 *                  &lt;entry delimiter=&quot;0x0a&quot;/&gt;
 *                  &lt;entry delimiter=&quot;0x09&quot;/&gt;
 *                  &lt;entry delimiter=&quot;0x0d&quot;/&gt;
 *             &lt;/delimiters&gt;
 *         &lt;/word-source-data&gt;
 *         &lt;word-index&gt;
 *             &lt;word&gt;&lt;![CDATA[Hello]]&gt;&lt;/word&gt;
 *             &lt;pos&gt;24&lt;/pos&gt;
 *             &lt;pos&gt;56&lt;/pos&gt;
 *         &lt;/word-index&gt;
 *     &lt;/document-index&gt;
 * &lt;/doc&gt;
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 *
 */
final class DocumentIndexHandler extends DefaultHandler {
    /**
     * <p>
     * Stores the locale information for this document index xml file.
     * It's initialized in constructor when the pass-in WordSourceId instance is not null or when the WordSourceId
     * instance is parsed from attributes of locale element.
     * </p>
     */
    private Locale locale;

    /**
     * <p>
     * Stores the source-identity element value in document index xml file.
     * Note, it may be null when the pass-in WordSourceId in constructor is not null.
     * </p>
     */
    private String encodedSourceIdentity;

    /**
     * <p>
     * Stores the class-name element value in document index xml file.
     * Note, it may be null when the pass-in WordSourceId in constructor is not null.
     * </p>
     */
    private String className;

    /**
     * <p>
     * Stores the delimiters information in document index xml file.
     * Note, it may be null when the pass-in WordSourceId in constructor is not null.
     * </p>
     */
    private List delimiterList;

    /**
     * <p>
     * Stores the words information in document index xml file.
     * The key is type of CollationKey, while the value is type of List, in which element is position integer.
     * </p>
     */
    private Map words;

    /**
     * <p>
     * Stores the document-id attribute value in document-index element.
     * </p>
     */
    private String documentIndexId;

    /**
     * <p>
     * Represents the current handled element name.
     * </p>
     */
    private String currentElementName;

    /**
     * <p>
     * Represents the WordSourceId contained in document index xml file.
     * Note, it may be parsed from xml file or assigned in the constructor.
     * When a not null WordSourceId instance is passed in, then the corresponding elements in xml file will be ignored.
     * </p>
     */
    private WordSourceId wordSourceId;

    /**
     * <p>
     * The current word is parsed.
     * Note, it's saved in form of CollationKey for language sensitive.
     * </p>
     */
    private CollationKey wordKey;

    /**
     * <p>
     * Represents the value of currentElementName.
     * Note, an element value may be splitted into several character parts, so this field is declared as a member to
     * concatenate all the characters.
     * </p>
     */
    private StringBuffer content = new StringBuffer();

    /**
     * <p>
     * Stores all the parsed positions that belongs to current wordKey.
     * </p>
     */
    private List positions;

    /**
     * <p>
     * Collator instance for getting CollationKey instance that relates to a source string.
     * It's initialized in constructor when the pass-in WordSourceId instance is not null or when the WordSourceId
     * instance is parsed from the current xml file.
     * </p>
     */
    private Collator collator;

    /**
     * <p>
     * Creates a new DocumentIndexHandler instance with WordSourceId specified.
     * </p>
     *
     * <p>
     * Note, the pass-in WordSourceId instance may be null. When it's null it means we have to parse the corresponding
     * elements related to WordSourceId, otherwise it means we can ignore the elements and treated this WordSourceId as
     * parsed from the xml file.
     * </p>
     *
     * @param wordSourceId the given WordSourceId instance that related to the current document index xml file to parse.
     * It may be null.
     * When it's null it means we have to parse the corresponding elements related to WordSourceId, otherwise it means
     * we can ignore the elements and treated this WordSourceId as parsed from the xml file.
     */
    DocumentIndexHandler(WordSourceId wordSourceId) {
        this.wordSourceId = wordSourceId;

        if (wordSourceId != null) {
            locale = wordSourceId.getSourceLocale();
            collator = Collator.getInstance(locale);
        } else {
            delimiterList = new ArrayList();
        }

        words = new HashMap();
    }

    /**
     * <p>
     * When the name of the current element is document-usage, collection, the corresponding document-id, usage-count
     * and collection-id attributes are parsed.
     * </p>
     *
     * <p>
     * Note, when the SAX parser encounters a star-tag, this method is called.
     * </p>
     *
     * @param namespaceURI namespace uri
     * @param localName local name
     * @param qName qualified name
     * @param attr represents the attributes for the current element encountered
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String namespaceURI, String localName,
        String qName, Attributes attr) throws SAXException {
        currentElementName = qName.intern();

        content.delete(0, content.length());

        // if wordSourceId is null then all the elements related to word-source-data
        // element will be parsed, otherwise they will be ignored
        if (wordSourceId == null) {
            if (Constants.DOCUMENT_INDEX == currentElementName) {
                //when document-index element is encountered, get the document-id attribute
                documentIndexId = attr.getValue(Constants.DOCUMENT_ID);
            } else if (Constants.LOCALE == currentElementName) {
                // when locale element is encountered, get the language, country and variant
                // attributes.
                String language = attr.getValue(Constants.LANGUAGE);
                String country = attr.getValue(Constants.COUNTRY);
                String variant = attr.getValue(Constants.VARIANT);

                // create a new Locale instance and Collator instance
                locale = new Locale(language, country, variant);
                collator = Collator.getInstance(locale);

            } else if (currentElementName == Constants.ENTRY) {
                // when entry element is encountered, get the delimiter attribute.
                // the delimiter will be decoded from its hex string representation
                String delimiter = attr.getValue(Constants.DELIMITER);
                delimiterList.add(DocumentIndexPersistenceUtil.hexDecode(delimiter));
            }
        }

        if (currentElementName == Constants.WORD_INDEX) {
            // if the start tag of word-index is encountered, then the positions list is cleared
            // to receive all the positions for the current word
            positions = new ArrayList();
        }
    }

    /**
     * <p>
     * If document-usage element ends, then currentDocumentId and the corresponding collectionList is stored to a map.
     * </p>
     *
     * <p>
     * Note, when the SAX parser encounters a end-tag, this method is called.
     * </p>
     *
     * @param namespaceURI namespace uri
     * @param localName local name
     * @param qName qualified name
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void endElement(String namespaceURI, String localName, String qName)
        throws SAXException {
        currentElementName = qName.intern();

        // if wordSourceId is null then all the elements related to word-source-data
        // element will be parsed, otherwise they will be ignored
        if (wordSourceId == null) {
            if (currentElementName == Constants.SOURCE_IDENTITY) {
                // when the end tag of source-identity element is encountered, save its element text value
                encodedSourceIdentity = content.toString();
            } else if (currentElementName == Constants.CLASS_NAME) {
                // when the end tag of class-name element is encountered, save its element text value
                className = content.toString();
            }
        }

        if (currentElementName == Constants.WORD) {
            // when the end tag of word element is encountered, save its element text value
            wordKey = collator.getCollationKey(content.toString());
        }

        if (currentElementName == Constants.POS) {
            // when the end tag of pos element is encountered, save its element text value
            positions.add(Integer.valueOf(content.toString()));
        }

        if (currentElementName == Constants.WORD_INDEX) {
            // when the end tag of word-index element is encountered, then a word information should be complete.
            words.put(wordKey, positions);
        }

        // clear environment
        content.delete(0, content.length());
        currentElementName = null;
    }

    /**
     * <p>
     * Buffers all the characters for the current element in the xml file.
     * </p>
     *
     * <p>
     * Note, when the SAX parser reads #PDATA, this method is called.
     * </p>
     *
     * @param ch The characters.
     * @param start The start position in the character array.
     * @param length The number of characters to use from the character array.
     *
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void characters(char[] ch, int start, int length)
        throws SAXException {
        //note, the text value for an element may be splitted to several characters
        //so, content buffer is used.
        content.append(ch, start, length);
    }

    /**
     * <p>
     * Returns the document index id for this document index xml file.
     * </p>
     *
     * @return the document index id for this document index xml file.
     */
    String getDocumentIndexId() {
        return documentIndexId;
    }

    /**
     * <p>
     * Returns all the words information contained in this document.
     * </p>
     *
     * <p>
     * The key is type of CollationKey, while the value is type of List, in which element is position integer.
     * </p>
     *
     * @return all the words information contained in this document.
     */
    Map getWords() {
        return words;
    }

    /**
     * <p>
     * Returns the WordSourceId instance for this document index.
     * </p>
     *
     * <p>
     * It may be directly assigned by constructor or parsed from the current document index file.
     * </p>
     *
     * @return the WordSourceId instance for this document index.
     * @throws IndexPersistenceException when failed to deserialize source identity object.
     */
    WordSourceId getWordSourceId() throws IndexPersistenceException {
        if (wordSourceId == null) {
            // if the wordSouceId is null, then it should be created using the parsed values from
            // document index file
            byte[] data = DocumentIndexPersistenceUtil.base64Decode(encodedSourceIdentity);
            Object sourceIdentity = DocumentIndexPersistenceUtil.deSerializeObject(data);
            String[] delimiters = (String[]) delimiterList.toArray(new String[delimiterList.size()]);
            this.wordSourceId = new WordSourceId(sourceIdentity, className,
                    delimiters, locale);
        }

        return wordSourceId;
    }
}
