/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.sql.Connection;

import java.util.Set;


/**
 * <p>
 * Unit test cases for AbstractDBIndexPersistence.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AbstractDBIndexPersistenceTests extends TestCase {
    /**
     * <p>
     * NAMESPACE used for testing.
     * </p>
     */
    private static final String NAMESPACE = "AbstractDBIndexPersistence";

    /**
     * <p>
     * AbstractDBIndexPersistence instance for testing.
     * </p>
     */
    private AbstractDBIndexPersistence persistence;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        TestHelper.loadMultipleXMLConfig(TestHelper.DB_FACTORY_NAMESPACE,
            TestHelper.DB_FACTORY_CONFIGFILE);
        persistence = new DummyDBIndexPersistence(TestHelper.getDBFactory(),
                TestHelper.SIMLE_ACCESS_CONNECTION_NAME);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void tearDown() throws Exception {
        TestHelper.clearConfigFile(TestHelper.DB_FACTORY_NAMESPACE);
        persistence = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(AbstractDBIndexPersistenceTests.class);
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory,String) for failure.
     * It tests the case that when connectionFactory is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1_NullConnectionFactory() throws Exception {
        try {
            new DummyDBIndexPersistence(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionName is null and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1_NullConnectionName() throws Exception {
        assertNotNull("Failed to create AbstractDBIndexPersistence instance with null connection name.",
            new DummyDBIndexPersistence(TestHelper.getDBFactory(), null));
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionName is empty and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor1_EmptyConnectionName() throws Exception {
        assertNotNull("Failed to create AbstractDBIndexPersistence instance with empty connection name.",
            new DummyDBIndexPersistence(TestHelper.getDBFactory(), " "));
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(String) for failure.
     * It tests the case that when namespace is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_NullNamespace() throws Exception {
        try {
            new DummyDBIndexPersistence(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(String) for failure.
     * It tests the case that when namespace is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_EmptyNamespace() throws Exception {
        try {
            new DummyDBIndexPersistence("");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(String) for failure.
     * It tests the case that when namespace doesn't exist in Configuration Manager and expects
     * PersistenceConfigurationException.
     * </p>
     */
    public void testCtor2_NotExistNamespace() {
        try {
            new DummyDBIndexPersistence("NotExistNamespace");
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionName is missing and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_ConnectionNameMissing() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/connectionNameMissing.xml");

        assertNotNull("Failed to create AbstractDBIndexPersistence instance when connection name is missing",
            new DummyDBIndexPersistence(NAMESPACE));

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(DBConnectionFactory,String) for accuracy.
     * It tests the case that when connectionFactoryClassName is missing and expects it success.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_ConnectionFactoryClassNameMissing()
        throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/connectionFactoryClassNameMissing.xml");

        assertNotNull(
            "Failed to create AbstractDBIndexPersistence instance when connection factory class name is missing",
            new DummyDBIndexPersistence(NAMESPACE));

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor AbstractDBIndexPersistence#AbstractDBIndexPersistence(String) for failure.
     * It tests the case when connectionFactoryClassNamespace is missing and expects for
     * PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_ConnectionFactoryClassNamespaceMissing()
        throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/connectionFactoryClassNamespaceMissing.xml");

        try {
            new DummyDBIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests AbstractDBIndexPersistence#getConnection() for accuracy.
     * It tests whether the returned connection has already been set auto-commit to false.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetConnection() throws Exception {
        Connection conn = persistence.getConnection();

        try {
            assertNotNull("Failed to get Connection instance.", conn);
            assertFalse("Failed to set auto commit to false.",
                conn.getAutoCommit());
        } finally {
            DBUtil.closeConnection(conn);
        }
    }

    /**
     * <p>
     * Tests AbstractDBIndexPersistence#beginTransaction(Connection) for failure.
     * It tests the case that when connection is null and expects IllegalArgumentException.
     * </p>
     */
    public void testBeginTransaction_NullConnection() {
        try {
            persistence.beginTransaction(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests AbstractDBIndexPersistence#commitTransaction(Connection) for failure.
     * It tests the case that when connection is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCommitTransaction_NullConnection()
        throws Exception {
        try {
            persistence.commitTransaction(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests AbstractDBIndexPersistence#rollbackTransaction(Connection) for failure.
     * It tests the case that when connection is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testRollbackTransaction_NullConnection()
        throws Exception {
        try {
            persistence.rollbackTransaction(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * This class extends AbstractDBIndexPersistence class.
     * It is used for testing propose.
     * </p>
     *
     * @author TCSDEVELOPER
     * @version 1.0
     */
    private static final class DummyDBIndexPersistence
        extends AbstractDBIndexPersistence {
        /**
         * <p>
         * Constructs a new DummyDBIndexPersistence instance.
         * </p>
         *
         * @param namespace The namespace for connection-related properties.
         * @throws IllegalArgumentException if namespace param is null or empty.
         * @throws PersistenceConfigurationException if there are configuration issues encountered
         */
        protected DummyDBIndexPersistence(String namespace)
            throws PersistenceConfigurationException {
            super(namespace);
        }

        /**
         * <p>
         * Constructs a new DummyDBIndexPersistence instance.
         * </p>
         *
         * @param connectionFactory DBConnectionFactory instance used to generate connections.
         * @param connectionName The name of the connection used for persistence operations, can be null.
         * @throws IllegalArgumentException if connectionFactory is null.
         */
        protected DummyDBIndexPersistence(
            DBConnectionFactory connectionFactory, String connectionName) {
            super(connectionFactory, connectionName);
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void addDocumentIndex(DocumentIndex arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @return null forever
         * @throws IndexPersistenceException never
         */
        public DocumentIndex getDocumentIndex(WordSourceId arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
            return null;
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void removeDocumentIndex(WordSourceId arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void addCollectionIndex(CollectionIndex arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @return null forever
         * @throws IndexPersistenceException never
         */
        public CollectionIndex getCollectionIndex(String arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
            return null;
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void removeCollectionIndex(String arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void updateCollectionIndex(CollectionIndex arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void increaseDocumentUseCount(WordSourceId arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @throws IndexPersistenceException never
         */
        public void decreaseDocumentUseCount(WordSourceId arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @param arg0 arg0
         * @return zero forever
         * @throws IndexPersistenceException never
         */
        public int getDocumentUseCount(WordSourceId arg0)
            throws IndexPersistenceException {
            // do nothing, testing only.
            return 0;
        }

        /**
         * <p>
         * Not implemented.
         * </p>
         *
         * @return null forever
         * @throws IndexPersistenceException never
         */
        public Set getIndexedDocuments() throws IndexPersistenceException {
            // do nothing, testing only.
            return null;
        }
    }
}
