/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.IndexManager;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSource;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.Set;


/**
 * <p>
 * Unit test cases for XmlIndexPersistence.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlIndexPersistenceTests extends TestCase {
    /**
     * <p>
     * NAMESPACE for configuration testing.
     * </p>
     */
    private static final String NAMESPACE = "XmlIndexPersistenceTests";

    /**
     * <p>
     * CollectionIndex Id constant for testing.
     * </p>
     */
    private static final String TEST_COLLECTION_ID = "93312458924";

    /**
     * <p>
     * XmlIndexPersistence instance for testing.
     * </p>
     */
    private XmlIndexPersistence persistence;

    /**
     * <p>
     * IndexManager instance for testing.
     * </p>
     */
    private IndexManager im;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void setUp() throws Exception {
        // set up necessary namespaces
        TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE,
            TestHelper.XML_INDEX_CONFIGFILE);

        persistence = new XmlIndexPersistence(TestHelper.XML_INDEX_NAMESPACE);
        im = new IndexManager(persistence);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void tearDown() throws Exception {
        // clear namespaces
        TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);

        // clear persistence path
        TestHelper.clearXmlSolutionFiles();

        persistence = null;
        im = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(XmlIndexPersistenceTests.class);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for accuracy.
     * </p>
     */
    public void testCtor1() {
        assertNotNull("Failed to create XmlIndexPersistence instance.",
            new XmlIndexPersistence(10, 100, "test_files/resource"));
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * It tests the case that when persistenceDirPath is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_NullPersistenceDirPath() {
        try {
            new XmlIndexPersistence(1, 100, null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * It tests the case that when persistenceDirPath is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor1_EmptyPersistenceDirPath() {
        try {
            new XmlIndexPersistence(1, 100, " ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * Expects for IllegalArgumentException.
     * </p>
     */
    public void testCtor1_XmlPersistencePathIsFile() {
        try {
            new XmlIndexPersistence(11, 23, "tests_fils/simple.txt");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * It tests the case when xmlPersistencePath represents a not existant file and expects for
     * IllegalArgumentException.
     * </p>
     */
    public void testCtor1_XmlPersistencePathNotExist() {
        try {
            new XmlIndexPersistence(11, 23, "tests/directory");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * It tests the case when fileSizeLimit is negative and expects for IllegalArgumentException.
     * </p>
     */
    public void testCtor1_FileSizeLimitNegative() {
        try {
            new XmlIndexPersistence(-11, 23, TestHelper.XML_PERSISTENCE_ROOT);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(int,int,String) for failure.
     * It tests the case when fileSizeThreshold is negative and expects for IllegalArgumentException.
     * </p>
     */
    public void testCtor1_FileSizeThresholdNegative() {
        try {
            new XmlIndexPersistence(1, -23, TestHelper.XML_PERSISTENCE_ROOT);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2() throws Exception {
        assertNotNull("Failed to create XmlIndexPersistence instance.",
            new XmlIndexPersistence(TestHelper.XML_INDEX_NAMESPACE));
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case that when namespace is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_NullNamespace() throws Exception {
        try {
            new XmlIndexPersistence(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case that when namespace is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_EmptyNamespace() throws Exception {
        try {
            new XmlIndexPersistence(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case when the given namespace is not configed in Configration Manager and expects for
     * PersistenceConfigurationException.
     * </p>
     *
     */
    public void testCtor2_NotExistNamespace() {
        try {
            new XmlIndexPersistence("NotExistNamespace");
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case when xmlPersistencePath property is missing and expects for
     * PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_XmlPersistencePathMissing() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/xmlPersistencePathMissing.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case when xmlPersitencePath is a file and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_XmlPersistencePathIsFile() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/xmlPersistencePathIsFile.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the case when xmlPersistencePath does not exist and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_XmlPersistencePathNotExist()
        throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/xmlPersistencePathNotExist.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the cases when fileSizeLimit property is missing and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCtor2_FileSizeLimitMissing() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/FileSizeLimitMissing.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the cases when fileSizeLimit property is negative and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testCtor2_FileSizeLimitNegative() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/FileSizeLimitNegative.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the cases when fileSizeThreshold property is missing and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testCtor2_FileSizeThresholdMissing() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/FileSizeThresholdMissing.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests ctor XmlIndexPersistence#XmlIndexPersistence(String) for failure.
     * It tests the cases when fileSizeThreshold property is negative and expects for PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testCtor2_FileSizeThresholdNegative() throws Exception {
        TestHelper.loadSingleXMLConfig(NAMESPACE,
            "test_files/test_configs/FileSizeThresholdNegative.xml");

        try {
            new XmlIndexPersistence(NAMESPACE);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(NAMESPACE);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addDocumentIndex(DocumentIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddDocumentIndex() throws Exception {
        DocumentIndex docIndex = new DocumentIndex(TestHelper.getMediumWordSource());
        int expectedWordsLength = docIndex.getWords().size();
        persistence.addDocumentIndex(docIndex);

        docIndex = persistence.getDocumentIndex(docIndex.getWordSourceId());

        int actualWordsLength = docIndex.getWords().size();
        assertEquals("Failed to get correct words for the given document index id.",
            expectedWordsLength, actualWordsLength);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addDocumentIndex(DocumentIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddDocumentIndexHuge() throws Exception {
        DocumentIndex docIndex = new DocumentIndex(TestHelper.getHugeWordSource());
        int expectedWordsLength = docIndex.getWords().size();
        persistence.addDocumentIndex(docIndex);

        docIndex = persistence.getDocumentIndex(docIndex.getWordSourceId());

        int actualWordsLength = docIndex.getWords().size();
        assertEquals("Failed to get correct words for the given document index id.",
            expectedWordsLength, actualWordsLength);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addDocumentIndex(DocumentIndex) for failure.
     * It tests the case that when documentIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddDocumentIndex_NullDocumentIndex()
        throws Exception {
        try {
            persistence.addDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addDocumentIndex(DocumentIndex) for failure.
     * It tests the case when trying to add a persisted document and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testAddDocumentIndex_IndexPersistenceException()
        throws Exception {
        try {
            DocumentIndex documentIndex = im.createDocumentIndex(TestHelper.getSimpleWordSource());

            persistence.addDocumentIndex(documentIndex);

            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getDocumentIndex(WordSourceId) for accuracy.
     * It tests the case when the wordSourceId hasn't been persisted and expects null.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentIndex() throws Exception {
        DocumentIndex index = persistence.getDocumentIndex(TestHelper.getSimpleWordSource()
                                                                     .getId());

        assertNull("The document index should not exist in persistence.", index);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getDocumentIndex(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentIndex_NullWordSourceId()
        throws Exception {
        try {
            persistence.getDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeDocumentIndex(WordSourceId) for accuracy.
     * Expects the indexed document removed successfully.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex() throws Exception {
        DocumentIndex docIndex = new DocumentIndex(TestHelper.getMediumWordSource());
        persistence.addDocumentIndex(docIndex);

        persistence.removeDocumentIndex(docIndex.getWordSourceId());

        assertNull("The document index should be deleted.",
            persistence.getDocumentIndex(docIndex.getWordSourceId()));
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeDocumentIndex(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex_NullWordSourceId()
        throws Exception {
        try {
            persistence.removeDocumentIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeDocumentIndex(WordSourceId) for failure.
     * It tests the case when trying to remove a not persisted WordSourceId and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveDocumentIndex_IndexPersistenceException()
        throws Exception {
        try {
            persistence.removeDocumentIndex(TestHelper.getSimpleWordSource()
                                                      .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addCollectionIndex(CollectionIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddCollectionIndex() throws Exception {
        im.createCollectionIndex(TEST_COLLECTION_ID);

        CollectionIndex collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertEquals("Expects equal collection id.", TEST_COLLECTION_ID,
            collectionIndex.getId());
        assertEquals("Expects zero document contained.", 0,
            collectionIndex.getAllDocumentIds().size());
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addCollectionIndex(CollectionIndex) for failure.
     * It tests the case that when collectionIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testAddCollectionIndex_NullCollectionIndex()
        throws Exception {
        try {
            persistence.addCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#addCollectionIndex(CollectionIndex) for failure.
     * It tests the case when trying to add a persisted WordSourceId and expects IndexPersistenceException.
     * </p>
     */
    public void testAddCollectionIndex_IndexPersistenceException() {
        try {
            CollectionIndex ci = im.createCollectionIndex(TEST_COLLECTION_ID);
            persistence.addCollectionIndex(ci);
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getCollectionIndex(String) for accuracy.
     * It tests the case when trying to get a not persisted document collection and expects null.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex() throws Exception {
        CollectionIndex collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertNull("The collection index should not exist in persistence.",
            collectionIndex);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getCollectionIndex(String) for failure.
     * It tests the case that when collectionId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex_NullCollectionId()
        throws Exception {
        try {
            persistence.getCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getCollectionIndex(String) for failure.
     * It tests the case that when collectionId is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetCollectionIndex_EmptyCollectionId()
        throws Exception {
        try {
            persistence.getCollectionIndex(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeCollectionIndex(String) for accuracy.
     * Expects the indexed document collection removed successfully.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex() throws Exception {
        //set up the database environment.
        im.createCollectionIndex(TEST_COLLECTION_ID);

        //remove it
        persistence.removeCollectionIndex(TEST_COLLECTION_ID);

        CollectionIndex ci = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertNull("The collection index should be removed.", ci);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case that when collectionId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex_NullCollectionId()
        throws Exception {
        try {
            persistence.removeCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case that when collectionId is empty and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testRemoveCollectionIndex_EmptyCollectionId()
        throws Exception {
        try {
            persistence.removeCollectionIndex(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#removeCollectionIndex(String) for failure.
     * It tests the case when trying to remove not persisted document collection and expects
     * IndexPersistenceException.
     * </p>
     */
    public void testRemoveCollectionIndex_IndexPersistenceException() {
        try {
            persistence.removeCollectionIndex("0001");
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#updateCollectionIndex(CollectionIndex) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testUpdateCollectionIndex() throws Exception {
        //set up the database environment.
        CollectionIndex collectionIndex = im.createCollectionIndex(TEST_COLLECTION_ID);

        //add a new document
        collectionIndex.addDocument(TestHelper.getSimpleWordSource());

        collectionIndex = persistence.getCollectionIndex(TEST_COLLECTION_ID);
        assertEquals("Expects equal collection id.", TEST_COLLECTION_ID,
            collectionIndex.getId());
        assertEquals("Expects one document contained.", 1,
            collectionIndex.getAllDocumentIds().size());
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#updateCollectionIndex(CollectionIndex) for failure.
     * It tests the case that when collectionIndex is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testUpdateCollectionIndex_NullCollectionIndex()
        throws Exception {
        try {
            persistence.updateCollectionIndex(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#updateCollectionIndex(CollectionIndex) for failure.
     * It tests the case when trying to update not persisted document collection and expects
     * IndexPersistenceException.
     * </p>
     */
    public void testUpdateCollectionIndex_IndexPersistenceException() {
        try {
            CollectionIndex ci = new CollectionIndex(im, TEST_COLLECTION_ID);
            persistence.updateCollectionIndex(ci);
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#increaseDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        im.createDocumentIndex(source);

        persistence.increaseDocumentUseCount(source.getId());

        int useCount = persistence.getDocumentUseCount(source.getId());
        assertEquals("Expects the value of use count is one.", 1, useCount);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#increaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.increaseDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#increaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to increase the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testIncreaseDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.increaseDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#decreaseDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        WordSourceId id = source.getId();
        im.createDocumentIndex(source);
        persistence.increaseDocumentUseCount(id);

        persistence.decreaseDocumentUseCount(id);

        int useCount = persistence.getDocumentUseCount(id);
        assertEquals("Expects the value of use count is zero.", 0, useCount);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#decreaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.decreaseDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#decreaseDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to decrease the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDecreaseDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.decreaseDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getDocumentUseCount(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount() throws Exception {
        WordSource source = TestHelper.getSimpleWordSource();
        im.createDocumentIndex(source);

        int useCount = persistence.getDocumentUseCount(source.getId());

        assertEquals("Expects the value of use count is zero.", 0, useCount);
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getDocumentUseCount(WordSourceId) for failure.
     * It tests the case that when wordSourceId is null and expects IllegalArgumentException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount_NullWordSourceId()
        throws Exception {
        try {
            persistence.getDocumentUseCount(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getDocumentUseCount(WordSourceId) for failure.
     * It tests the case when trying to get the use count of a not persisted document and expects
     * IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDocumentUseCount_IndexPersistenceException()
        throws Exception {
        try {
            persistence.getDocumentUseCount(TestHelper.getSimpleWordSource()
                                                           .getId());
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests XmlIndexPersistence#getIndexedDocuments() for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetIndexedDocuments() throws Exception {
        im.createDocumentIndex(TestHelper.getMediumWordSource());
        im.createDocumentIndex(TestHelper.getSimpleWordSource());

        Set documents = persistence.getIndexedDocuments();

        assertEquals("The returned set should contain two elements.", 2,
            documents.size());
    }
}
